'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var DEFAULT_ALLOW_METHODS = ['POST', 'GET', 'PUT', 'PATCH', 'DELETE', 'OPTIONS'];

var DEFAULT_ALLOW_HEADERS = ['X-Requested-With', 'Access-Control-Allow-Origin', 'X-HTTP-Method-Override', 'Content-Type', 'Authorization', 'Accept'];

var DEFAULT_MAX_AGE_SECONDS = 60 * 60 * 24; // 24 hours

function isString(s) {
  return typeof s === 'string' || s instanceof String;
}

function isOriginAllowed(origin, allowedOrigin) {
  if (Array.isArray(allowedOrigin)) {
    for (var i in allowedOrigin) {
      if (isOriginAllowed(origin, allowedOrigin[i])) {
        return true;
      }
    }
  } else if (isString(allowedOrigin)) {
    return origin === allowedOrigin;
  } else if (allowedOrigin instanceof RegExp) {
    return allowedOrigin.test(origin);
  }
  return false;
}

function setVaryHeader(res, origin) {
  if (!isString(origin)) {
    if (res.getHeader('Vary')) {
      res.setHeader('Vary', res.getHeader('Vary') + ',Origin');
    } else {
      res.setHeader('Vary', 'Origin');
    }
  }
}

var cors = function cors() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return function (handler) {
    return function () {
      var _ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee(req, res) {
        for (var _len = arguments.length, restArgs = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
          restArgs[_key - 2] = arguments[_key];
        }

        var _options$origin, origin, _options$maxAge, maxAge, _options$allowMethods, allowMethods, _options$allowHeaders, allowHeaders, _options$allowCredent, allowCredentials, _options$exposeHeader, exposeHeaders, _options$runHandlerOn, runHandlerOnOptionsRequest, preFlight, handlerResult;

        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _options$origin = options.origin, origin = _options$origin === undefined ? '*' : _options$origin, _options$maxAge = options.maxAge, maxAge = _options$maxAge === undefined ? DEFAULT_MAX_AGE_SECONDS : _options$maxAge, _options$allowMethods = options.allowMethods, allowMethods = _options$allowMethods === undefined ? DEFAULT_ALLOW_METHODS : _options$allowMethods, _options$allowHeaders = options.allowHeaders, allowHeaders = _options$allowHeaders === undefined ? DEFAULT_ALLOW_HEADERS : _options$allowHeaders, _options$allowCredent = options.allowCredentials, allowCredentials = _options$allowCredent === undefined ? true : _options$allowCredent, _options$exposeHeader = options.exposeHeaders, exposeHeaders = _options$exposeHeader === undefined ? [] : _options$exposeHeader, _options$runHandlerOn = options.runHandlerOnOptionsRequest, runHandlerOnOptionsRequest = _options$runHandlerOn === undefined ? false : _options$runHandlerOn;

                if (req.headers.origin) {
                  _context.next = 3;
                  break;
                }

                return _context.abrupt('return', handler.apply(undefined, [req, res].concat(restArgs)));

              case 3:

                if (isString(origin)) {
                  res.setHeader('Access-Control-Allow-Origin', origin);
                } else {
                  if (isOriginAllowed(req.headers.origin, origin)) {
                    res.setHeader('Access-Control-Allow-Origin', req.headers.origin);
                  }
                }

                if (allowCredentials) {
                  res.setHeader('Access-Control-Allow-Credentials', 'true');
                }
                if (exposeHeaders.length) {
                  res.setHeader('Access-Control-Expose-Headers', exposeHeaders.join(','));
                }

                preFlight = req.method === 'OPTIONS';

                if (preFlight) {
                  res.setHeader('Access-Control-Allow-Methods', allowMethods.join(','));
                  res.setHeader('Access-Control-Allow-Headers', allowHeaders.join(','));
                  res.setHeader('Access-Control-Max-Age', String(maxAge));
                }

                if (!(preFlight && !runHandlerOnOptionsRequest)) {
                  _context.next = 13;
                  break;
                }

                setVaryHeader(res, origin);
                res.end();
                _context.next = 18;
                break;

              case 13:
                _context.next = 15;
                return handler.apply(undefined, [req, res].concat(restArgs));

              case 15:
                handlerResult = _context.sent;

                setVaryHeader(res, origin);
                return _context.abrupt('return', handlerResult);

              case 18:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, undefined);
      }));

      return function (_x2, _x3) {
        return _ref.apply(this, arguments);
      };
    }();
  };
};

module.exports = cors;